@@ -2,16 +2,17 @@ from concurrent.futures import ThreadPoolExecutor, as_completed
 import itertools
 
 from numpy import timedelta64
+from ldimbenchmark.benchmark.results import load_result
 from ldimbenchmark.benchmark.runners import DockerMethodRunner, LocalMethodRunner
 from ldimbenchmark.benchmark.runners.BaseMethodRunner import MethodRunner
 from ldimbenchmark.datasets import Dataset
 import pandas as pd
+import numpy as np
 from typing import Dict, Literal, TypedDict, Union, List, Callable
 import os
 import logging
-from ldimbenchmark.constants import LDIM_BENCHMARK_CACHE_DIR
+from ldimbenchmark.constants import CPU_COUNT, LDIM_BENCHMARK_CACHE_DIR
 from glob import glob
-from ldimbenchmark.benchmark_evaluation import evaluate_leakages
 from tabulate import tabulate
 from ldimbenchmark.benchmark_complexity import run_benchmark_complexity
 import matplotlib.pyplot as plt
@@ -29,6 +30,11 @@ from sqlalchemy import create_engine
 import matplotlib.pyplot as plt
 import matplotlib.dates as mdates
 from matplotlib import patches
+import ast
+import seaborn as sns
+from matplotlib.ticker import FormatStrFormatter
+
+from ldimbenchmark.utilities import get_method_name_from_docker_image
 
 
 def execute_experiment(experiment: MethodRunner):
@@ -49,8 +55,9 @@ def plot_leak(
     leak_pair,
     additional_data_dir,
     out_dir,
+    boundary_timespan_overwrite: pd.Timedelta = None,
 ):
-    fig = plt.figure()
+    fig = plt.figure(figsize=(20, 10))
     gs = fig.add_gridspec(3, hspace=0)
     ax_dataset_flows, ax_dataset_pressures, ax_method = gs.subplots(
         sharex=True, sharey=False
@@ -59,45 +66,52 @@ def plot_leak(
     name = ""
     expected_leak, detected_leak = leak_pair
 
-    reference_leak = None
-    boundaries = None
+    boundary = None
+    reference_time = None
+    leak_time = pd.Timedelta(0)
     if expected_leak is not None:
-        name = str(expected_leak["leak_time_start"])
-        reference_leak = expected_leak
-        boundaries = (
-            expected_leak["leak_time_end"] - expected_leak["leak_time_start"]
-        ) / 6
+        expected_leak_start = expected_leak["leak_time_start"]
+        expected_leak_end = expected_leak["leak_time_end"]
+        name = expected_leak_start.strftime("%Y_%m_%d_%H_%M_%S")
+
+        if pd.isna(expected_leak_end):
+            expected_leak_end = expected_leak["leak_time_start"]
+        if expected_leak_start != expected_leak_end:
+            boundary = (expected_leak_end - expected_leak_start) / 3
+        reference_time = expected_leak_start
+        leak_time = expected_leak_end - expected_leak_start
+
+        if expected_leak_start == expected_leak_end:
+            ax_dataset_flows.axvline(expected_leak_start, color="red", zorder=4)
+            ax_dataset_pressures.axvline(expected_leak_start, color="red", zorder=4)
+            ax_method.axvline(expected_leak_start, color="red", zorder=4)
+        else:
+            ax_dataset_flows.axvspan(
+                expected_leak_start,
+                expected_leak_end,
+                color="red",
+                alpha=0.1,
+                lw=0,
+                zorder=1,
+            )
+            ax_dataset_pressures.axvspan(
+                expected_leak_start,
+                expected_leak_end,
+                color="red",
+                alpha=0.1,
+                lw=0,
+                zorder=1,
+            )
+            ax_method.axvspan(
+                expected_leak_start,
+                expected_leak_end,
+                color="red",
+                alpha=0.1,
+                lw=0,
+                zorder=1,
+            )
+        ax_method.text(expected_leak_start, 0, "Expected Leak", rotation=90)
 
-        ax_dataset_flows.axvspan(
-            expected_leak["leak_time_start"],
-            expected_leak["leak_time_end"]
-            if not pd.isna(expected_leak["leak_time_end"])
-            else expected_leak["leak_time_start"],
-            color="red",
-            alpha=0.1,
-            lw=0,
-            zorder=1,
-        )
-        ax_dataset_pressures.axvspan(
-            expected_leak["leak_time_start"],
-            expected_leak["leak_time_end"]
-            if not pd.isna(expected_leak["leak_time_end"])
-            else expected_leak["leak_time_start"],
-            color="red",
-            alpha=0.1,
-            lw=0,
-            zorder=1,
-        )
-        ax_method.axvspan(
-            expected_leak["leak_time_start"],
-            expected_leak["leak_time_end"]
-            if not pd.isna(expected_leak["leak_time_end"])
-            else expected_leak["leak_time_start"],
-            color="red",
-            alpha=0.1,
-            lw=0,
-            zorder=1,
-        )
     # Plot detected leak:
     if detected_leak is not None:
         ax_dataset_flows.axvline(
@@ -107,32 +121,36 @@ def plot_leak(
             detected_leak["leak_time_start"], color="green", zorder=4
         )
         ax_method.axvline(detected_leak["leak_time_start"], color="green", zorder=4)
+        ax_method.text(
+            detected_leak["leak_time_start"], 0, "Detected Leak", rotation=90
+        )
 
     #
     if expected_leak is None and detected_leak is not None:
-        name = str(detected_leak["leak_time_start"]) + "_fp"
-        reference_leak = detected_leak
+        name = detected_leak["leak_time_start"].strftime("%Y_%m_%d_%H_%M_%S") + "_fp"
+        reference_time = detected_leak["leak_time_start"]
 
     # Plot expected leak:
     if detected_leak is None and expected_leak is not None:
-        name = str(expected_leak["leak_time_start"]) + "_fn"
+        name = expected_leak["leak_time_start"].strftime("%Y_%m_%d_%H_%M_%S") + "_fn"
 
+    ax_dataset_pressures.set_ylabel("Pressure")
     for sensor_id, sensor_readings in getattr(dataset, "pressures").items():
-        if boundaries == None:
+        if boundary == None:
             # Just use first sensor_readings for all...
-            boundaries = (sensor_readings.index[-1] - sensor_readings.index[0]) / (
+            boundary = (sensor_readings.index[-1] - sensor_readings.index[0]) / (
                 sensor_readings.shape[0] / 6
             )
             minimum_boundary = timedelta64(1, "D")
-            if boundaries < minimum_boundary:
-                boundaries = minimum_boundary
+            if boundary < minimum_boundary:
+                boundary = minimum_boundary
+        if boundary_timespan_overwrite is not None:
+            boundary = boundary_timespan_overwrite
         mask = get_mask(
             sensor_readings,
-            reference_leak["leak_time_start"],
-            reference_leak["leak_time_end"]
-            if not pd.isna(reference_leak["leak_time_end"])
-            else reference_leak["leak_time_start"],
-            boundaries,
+            reference_time,
+            reference_time + leak_time,
+            boundary,
         )
 
         sensor_readings = sensor_readings[mask]
@@ -145,23 +163,23 @@ def plot_leak(
             zorder=3,
             label=sensor_id,
         )
-        # sensor_readings.plot(ax=ax, alpha=0.2, linestyle="solid", zorder=3)
+    ax_dataset_flows.set_ylabel("Flow")
     for sensor_id, sensor_readings in getattr(dataset, "flows").items():
-        if boundaries == None:
+        if boundary == None:
             # Just use first sensor_readings for all...
-            boundaries = (sensor_readings.index[-1] - sensor_readings.index[0]) / (
+            boundary = (sensor_readings.index[-1] - sensor_readings.index[0]) / (
                 sensor_readings.shape[0] / 6
             )
             minimum_boundary = timedelta64(1, "D")
-            if boundaries < minimum_boundary:
-                boundaries = minimum_boundary
+            if boundary < minimum_boundary:
+                boundary = minimum_boundary
+        if boundary_timespan_overwrite is not None:
+            boundary = boundary_timespan_overwrite
         mask = get_mask(
             sensor_readings,
-            reference_leak["leak_time_start"],
-            reference_leak["leak_time_end"]
-            if not pd.isna(reference_leak["leak_time_end"])
-            else reference_leak["leak_time_start"],
-            boundaries,
+            reference_time,
+            reference_time + leak_time,
+            boundary,
         )
 
         sensor_readings = sensor_readings[mask]
@@ -178,22 +196,24 @@ def plot_leak(
     # Plot debug data:
     debug_folder = os.path.join(additional_data_dir, "debug/")
     # TODO: Adjust Mask for each debug data
+    ax_method.set_ylabel("Debug")
+    ax_method.set_xlabel("Time")
     if os.path.exists(debug_folder):
         files = glob(debug_folder + "*.csv")
         for file in files:
             try:
                 debug_data = pd.read_csv(file, parse_dates=True, index_col=0)
-                if boundaries == None:
+                if boundary == None:
                     alternative_boundarys = (
                         sensor_readings.index[-1] - sensor_readings.index[0]
                     ) / (sensor_readings.shape[0] / 6)
+                if boundary_timespan_overwrite is not None:
+                    boundary = boundary_timespan_overwrite
                 mask = get_mask(
                     debug_data,
-                    reference_leak["leak_time_start"],
-                    reference_leak["leak_time_end"]
-                    if not pd.isna(reference_leak["leak_time_end"])
-                    else reference_leak["leak_time_start"],
-                    boundaries,
+                    reference_time,
+                    reference_time + leak_time,
+                    boundary,
                 )
                 debug_data = debug_data[mask]
                 for column in debug_data.columns:
@@ -216,6 +236,9 @@ def plot_leak(
     ax_method.set_title("Debug Data from Method", y=1.0, pad=-14)
     ax_dataset_pressures.set_title("Pressure Data From Dataset", y=1.0, pad=-14)
     ax_dataset_flows.set_title("Flows Data From Dataset", y=1.0, pad=-14)
+
+    date_form = mdates.DateFormatter("%Y-%m-%d %H:%M")
+    ax_method.xaxis.set_major_formatter(date_form)
     # TODO: Plot Leak Outflow, if available
 
     # Put a legend to the right of the current axis
@@ -225,42 +248,55 @@ def plot_leak(
     plt.close(fig)
 
 
-def load_result(folder: str) -> Dict:
-    folder = os.path.join(folder, "")
-    detected_leaks_file = os.path.join(folder, "detected_leaks.csv")
-    if not os.path.exists(detected_leaks_file):
-        return {}
+def create_plots(
+    results: pd.DataFrame,
+    method: str,
+    dataset: str,
+    hyperparameters: List[str],
+    performance_metric: str,
+    out_folder,
+):
+    plot_out_folder = os.path.join(out_folder)  # , method, dataset)
+    os.makedirs(plot_out_folder, exist_ok=True)
+    plot_data = results[(results["method"] == method) & (results["dataset"] == dataset)]
+    hyperparameters = list(map(lambda x: "hyperparameters." + x, hyperparameters))
 
-    detected_leaks = pd.read_csv(
-        detected_leaks_file,
-        parse_dates=True,
-        date_parser=lambda x: pd.to_datetime(x, utc=True),
+    hyperparameter_combination = list(itertools.combinations(hyperparameters, 2))
+
+    cmap = sns.cm.rocket_r
+    fig, axs = plt.subplots(
+        ncols=len(hyperparameters) - 1,
+        nrows=len(hyperparameters) - 1,
+        figsize=(len(hyperparameters) * 4, len(hyperparameters) * 4),
+        squeeze=False,
     )
+    for row, param_1 in enumerate(hyperparameters):
+        for col, param_2 in enumerate(hyperparameters):
+            if (param_2, param_1) in hyperparameter_combination:
+                real_row = row - 1
+                pvt = pd.pivot_table(
+                    plot_data,
+                    values=performance_metric,
+                    index=param_1,
+                    columns=param_2,  # , "hyperparameters.delta"]
+                )
+                if len(pvt) != 0:
+                    sns.heatmap(pvt, ax=axs[real_row, col], cmap=cmap)
+                axs[real_row, col].set_ylabel(param_1)
+                axs[real_row, col].set_xlabel(param_2)
+                # axs[real_row, col].yaxis.set_major_formatter(FormatStrFormatter("%.2f"))
+                axs[real_row, col].xaxis.set_major_formatter(FormatStrFormatter("%.2f"))
 
-    evaluation_dataset_leakages = pd.read_csv(
-        os.path.join(folder, "should_have_detected_leaks.csv"),
-        parse_dates=True,
-        date_parser=lambda x: pd.to_datetime(x, utc=True),
-    )
-
-    run_info = pd.read_csv(os.path.join(folder, "run_info.csv")).iloc[0]
-
-    # TODO: Ignore Detections outside of the evaluation period
-    (evaluation_results, matched_list) = evaluate_leakages(
-        evaluation_dataset_leakages, detected_leaks
-    )
-    evaluation_results["method"] = run_info["method"]
-    evaluation_results["method_version"] = run_info.get("method_version", None)
-    evaluation_results["dataset"] = run_info["dataset"]
-    evaluation_results["dataset_part"] = run_info.get("dataset_part", None)
-    evaluation_results["dataset_id"] = run_info["dataset_id"]
-    evaluation_results["dataset_derivations"] = run_info["dataset_options"]
-    evaluation_results["hyperparameters"] = run_info["hyperparameters"]
-    evaluation_results["matched_leaks_list"] = matched_list
-    evaluation_results["_folder"] = os.path.basename(os.path.dirname(folder))
-    evaluation_results["executed_at"] = run_info.get("executed_at", None)
-
-    return evaluation_results
+            else:
+                if len(hyperparameters) > 2 and (
+                    row + (len(hyperparameters) - col) < (len(hyperparameters) - 1)
+                ):
+                    axs[row, col - 1].set_axis_off()
+    fig.suptitle(f"Heatmaps for {method}-{dataset}", fontsize=16)
+    fig.tight_layout()
+    fig.subplots_adjust(top=0.88)
+    fig.savefig(os.path.join(plot_out_folder, f"heatmap_{method}_{dataset}.png"))
+    plt.close(fig)
 
 
 # TODO: Draw plots with leaks and detected leaks
@@ -401,10 +437,15 @@ class LDIMBenchmark:
                             #     **hyperparameters_method_map[method_id][key],
                             # }
             else:
-                for dataset_base_id in dataset_base_ids:
-                    hyperparameters_map[method_id][
-                        dataset_base_id
-                    ] = hyperparameters_method_map[method_id]
+                if len(dataset_base_ids) == 0:
+                    hyperparameters_map[method_id] = hyperparameters_method_map[
+                        method_id
+                    ]
+                else:
+                    for dataset_base_id in dataset_base_ids:
+                        hyperparameters_map[method_id][
+                            dataset_base_id
+                        ] = hyperparameters_method_map[method_id]
 
         # if method_id in hyperparameters:
         #     if dataset_base_id in hyperparameters[method_id]:
@@ -456,25 +497,30 @@ class LDIMBenchmark:
         self,
         methods,
         style: Literal["time", "junctions"],
+        n_repeats=3,
+        n_measures=10,
+        n_max=91,
     ):
         complexity_results_path = os.path.join(self.complexity_results_dir, style)
         os.makedirs(complexity_results_path, exist_ok=True)
-        if style == "time":
-            return run_benchmark_complexity(
-                methods,
-                cache_dir=os.path.join(self.cache_dir, "datagen"),
-                out_folder=complexity_results_path,
-                style="time",
-                additionalOutput=self.debug,
-            )
-        if style == "junctions":
-            return run_benchmark_complexity(
-                methods,
-                cache_dir=os.path.join(self.cache_dir, "datagen"),
-                out_folder=complexity_results_path,
-                style="junctions",
-                additionalOutput=self.debug,
-            )
+        hyperparameters_map = self._get_hyperparameters_for_methods_and_datasets(
+            hyperparameters=self.hyperparameters,
+            method_ids=[
+                get_method_name_from_docker_image(lmethod) for lmethod in methods
+            ],
+            dataset_base_ids=[],
+        )
+        return run_benchmark_complexity(
+            methods,
+            cache_dir=os.path.join(self.cache_dir, "datagen"),
+            out_folder=complexity_results_path,
+            style=style,
+            additionalOutput=self.debug,
+            hyperparameters=hyperparameters_map,
+            n_repeats=n_repeats,
+            n_measures=n_measures,
+            n_max=n_max,
+        )
 
     def run_benchmark(
         self,
@@ -503,7 +549,8 @@ class LDIMBenchmark:
         hyperparameters_map = self._get_hyperparameters_for_methods_and_datasets(
             hyperparameters=self.hyperparameters,
             method_ids=[
-                dmethod.split(":")[0].split("/")[-1] for dmethod in self.methods_docker
+                get_method_name_from_docker_image(dmethod)
+                for dmethod in self.methods_docker
             ]
             + [lmethod.name for lmethod in self.methods_local],
             dataset_base_ids=[dataset.id for dataset in self.datasets],
@@ -578,7 +625,6 @@ class LDIMBenchmark:
                 fill="-",
                 justify=enlighten.Justify.CENTER,
             )
-        results = []
         bar_experiments = manager.counter(
             total=num_experiments,
             desc="Experiments",
@@ -588,7 +634,7 @@ class LDIMBenchmark:
         bar_experiments.refresh()
         # This line makes sure we can call update with an effect
         if parallel:
-            worker_num = os.cpu_count() - 1
+            worker_num = CPU_COUNT
             if parallel_max_workers > 0:
                 worker_num = parallel_max_workers
             try:
@@ -609,7 +655,7 @@ class LDIMBenchmark:
                 manager.stop()
         else:
             for experiment in self.experiments:
-                results.append(experiment.run())
+                experiment.run()
                 bar_experiments.update()
         manager.stop()
 
@@ -617,6 +663,7 @@ class LDIMBenchmark:
         self,
         current_only=True,
         resultFilter: Callable = lambda r: r,
+        print_results: bool = True,
         write_results: Union[None, Literal["csv", "db"]] = None,
         evaluations: List[Callable] = [
             precision,
@@ -674,7 +721,7 @@ class LDIMBenchmark:
             unit="results",
         )
         results = []
-        parallel = True
+        parallel = False
         if parallel == True:
             with ProcessPoolExecutor() as executor:
                 # submit all tasks and get future objects
@@ -744,13 +791,40 @@ class LDIMBenchmark:
                         errors="ignore",
                     )
                     leak_pairs["result_id"] = index
-                    leak_pairs.to_sql("leak_pairs", engine, if_exists="append")
+                    leak_pairs.to_sql("leak_pairs", engine, if_exists="replace")
             except Exception as e:
                 print(e)
                 print(f"Could not write leak pairs to database. For {index}")
             results = results.drop(columns=["matched_leaks_list"])
             results.to_sql("results", engine, if_exists="replace")
 
+        # Generate Heatmaps if multiple parameters are used
+        if self.multi_parameters:
+            results.hyperparameters = results.hyperparameters.astype("str")
+            df_hyperparameters = pd.json_normalize(
+                results.hyperparameters.apply(ast.literal_eval)
+            ).add_prefix("hyperparameters.")
+            df_hyperparameters.index = results.index
+            df_hyperparameters
+            # results = results.drop(columns=["hyperparameters"])
+            flat_results = pd.concat([results, df_hyperparameters], axis=1)
+
+            performance_indicator = "F1"
+
+            for method in list(
+                map(lambda x: x.name, self.methods_docker + self.methods_local)
+            ):
+                for dataset_name in map(lambda x: x.name, self.datasets):
+                    logging.info(f"Generating Heatmap for {method} {dataset_name}")
+                    create_plots(
+                        flat_results,
+                        method,
+                        dataset_name,
+                        self.hyperparameters[method].keys(),
+                        performance_indicator,
+                        self.evaluation_results_dir,
+                    )
+
         results = results.set_index(["method", "method_version", "dataset_id"])
         results = results.sort_values(by=["F1"])
         # Display in console
@@ -779,18 +853,14 @@ class LDIMBenchmark:
             "F1",
         ]
         results.columns = columns
-
-        print(tabulate(results, headers="keys"))
+        if print_results:
+            print(tabulate(results, headers="keys"))
         return results
 
-    def evaluate_run(
-        self,
-        run_id: str,
-    ):
+    def evaluate_run(self, run_id: str, boundary_timespan_overwrite=None):
         result_folder = os.path.join(self.runner_results_dir, run_id)
         result = load_result(result_folder)
 
-        logging.info("Generating plots per leak ...")
         manager = enlighten.get_manager()
         loaded_datasets = {}
         for dataset in self.datasets:
@@ -805,43 +875,6 @@ class LDIMBenchmark:
         graph_dir = os.path.join(self.evaluation_results_dir, "per_run", run_id)
         os.makedirs(graph_dir, exist_ok=True)
 
-        pbar2 = manager.counter(
-            total=len(result["matched_leaks_list"]),
-            desc="Graphs:",
-            unit="graphs",
-        )
-        parallel = True
-        if parallel:
-            with ProcessPoolExecutor() as executor:
-                # submit all tasks and get future objects
-                futures = []
-                for leak_pair in result["matched_leaks_list"]:
-                    future = executor.submit(
-                        plot_leak,
-                        loaded_datasets[result["dataset_id"]],
-                        leak_pair=leak_pair,
-                        additional_data_dir=result_folder,
-                        out_dir=graph_dir,
-                    )
-                    futures.append(future)
-
-                # process results from tasks in order of task completion
-                for future in as_completed(futures):
-                    future.result()
-                    pbar2.update()
-
-        else:
-            for leak_pair in result["matched_leaks_list"]:
-                plot_leak(
-                    getattr(loaded_datasets[result["dataset_id"]], "pressures"),
-                    leak_pair=leak_pair,
-                    additional_data_dir=result_folder,
-                    out_dir=graph_dir,
-                )
-                pbar2.update()
-        pbar2.close()
-        manager.stop()
-
         # Generate Leak Overview
         fig, ax = plt.subplots(figsize=(10, 10))
         ax.set_axisbelow(True)
@@ -858,7 +891,7 @@ class LDIMBenchmark:
                     expected_leak["leak_time_end"]
                 ) - mdates.date2num(expected_leak["leak_time_start"])
                 if length <= 0:
-                    length = 1
+                    length = 2
                 ax.barh(
                     [num],
                     # [2],
@@ -886,7 +919,7 @@ class LDIMBenchmark:
                 )
         ax.set_yticks(range(1, len(result["matched_leaks_list"]) + 1))
         # ax.xaxis_date()
-        ax.set_title("Leak overview")
+        ax.set_title("leak time spans overview")
         ax.set_ylabel("Leaks")
         ax.set_xlabel("Time")
         yellow_patch = patches.Patch(color="yellow", label="Expected Leaks")
@@ -896,12 +929,48 @@ class LDIMBenchmark:
         fig.savefig(os.path.join(graph_dir, "leaks_overview.png"))
         plt.close(fig)
 
+        logging.info("Generating plots per leak ...")
+        pbar2 = manager.counter(
+            total=len(result["matched_leaks_list"]),
+            desc="Graphs:",
+            unit="graphs",
+        )
+        parallel = False
+        if parallel:
+            with ProcessPoolExecutor(max_workers=CPU_COUNT) as executor:
+                # submit all tasks and get future objects
+                futures = []
+                for leak_pair in result["matched_leaks_list"]:
+                    future = executor.submit(
+                        plot_leak,
+                        loaded_datasets[result["dataset_id"]],
+                        leak_pair=leak_pair,
+                        additional_data_dir=result_folder,
+                        out_dir=graph_dir,
+                        boundary_timespan_overwrite=boundary_timespan_overwrite,
+                    )
+                    futures.append(future)
+
+                # process results from tasks in order of task completion
+                for future in as_completed(futures):
+                    future.result()
+                    pbar2.update()
+
+        else:
+            for leak_pair in result["matched_leaks_list"]:
+                plot_leak(
+                    loaded_datasets[result["dataset_id"]],
+                    leak_pair=leak_pair,
+                    additional_data_dir=result_folder,
+                    out_dir=graph_dir,
+                )
+                pbar2.update()
+        pbar2.close()
+        manager.stop()
+
         statistics_table = []
 
         pd.DataFrame(statistics_table)
 
         # TODO: Statistics about the leaks
         # Which leaks are detected? short/long, which leaks are not detected?
-
-
-# TODO: Generate overlaying graphs of leak size and detection times (and additional output)
