@@ -1,5 +1,5 @@
 from .lila import LILA
 from .mnf import MNF
 from .dualmethod import DUALMethod
-from .null import NullLeakagedDetectionMethod
+from .null import NullLeakageDetectionMethod
 from .random import RandomMethod
